@@ -1,4 +1,5 @@
 package HTML::KhatGallery::Core;
+$HTML::KhatGallery::Core::VERSION = '0.20';
 use strict;
 use warnings;
 
@@ -8,11 +9,7 @@ HTML::KhatGallery::Core - the core methods for HTML::KhatGallery
 
 =head1 VERSION
 
-This describes version B<0.03> of HTML::KhatGallery::Core.
-
-=cut
-
-our $VERSION = '0.03';
+version 0.20
 
 =head1 SYNOPSIS
 
@@ -42,7 +39,7 @@ use File::Spec;
 use Cwd;
 use File::stat;
 use YAML qw(Dump LoadFile);
-use Image::Info qw(image_info dim);
+use Image::ExifTool;
 
 =head1 CLASS METHODS
 
@@ -153,9 +150,19 @@ of the thumbnails, but their area.  This gives better-quality thumbnails.
 
 =item B<top_dir>
 
-The directory to create galleries in; this will be searched for
-images and sub-directories, and HTML and thumbnails will be created
-there.  If this is not given, the current directory is used.
+The directory to look for images in; this will be searched for images and
+sub-directories.  If this is not given, the current directory is used.
+
+=item B<top_out_dir>
+
+The directory to create galleries in; HTML and thumbnails will be created
+there.  If this is not given, it is the same as B<top_dir>.
+
+=item B<top_url>
+
+The URL of the top images directory; if the top_out_dir isn't the
+same as the top_dir, then we need to know this in order
+to link to the images in the images directory.
 
 =item B<verbose>
 
@@ -226,11 +233,26 @@ sub init {
     {
 	$self->{top_dir} = '.';
     }
-    # make top_dir absolute
     $self->{top_dir} = File::Spec->rel2abs($self->{top_dir});
-    # get the basename of the top dir
     $self->{top_base} = basename($self->{top_dir});
 
+    # top_out_dir
+    if (!defined $self->{top_out_dir})
+    {
+	$self->{top_out_dir} = $self->{top_dir};
+    }
+    $self->{top_out_dir} = File::Spec->rel2abs($self->{top_out_dir});
+
+    # trim top_url if it has a trailing slash
+    if (defined $self->{top_url})
+    {
+	$self->{top_url} =~ s!/$!!;
+    }
+    else
+    {
+	$self->{top_url} = '';
+    }
+
     # calculate width and height of thumbnail display
     $self->{thumb_geom} =~ /(\d+)x(\d+)/;
     $self->{thumb_width} = $1;
@@ -242,6 +264,7 @@ sub init {
 	$self->{dir_actions} = [qw(init_settings
 	    read_captions
 	    read_dir
+	    read_out_dir
 	    filter_images
 	    sort_images
 	    filter_dirs
@@ -399,19 +422,22 @@ sub init_settings {
     my $dir_state = shift;
 
     $dir_state->{abs_dir} = File::Spec->catdir($self->{top_dir}, $dir_state->{dir});
+    $dir_state->{abs_out_dir} = File::Spec->catdir($self->{top_out_dir}, $dir_state->{dir});
     my @path = File::Spec->splitdir($dir_state->{abs_dir});
     if ($dir_state->{dir})
     {
 	$dir_state->{dirbase} = pop @path;
 	$dir_state->{parent} = pop @path;
+	$dir_state->{dir_url} = $self->{top_url} . '/' . $dir_state->{dir};
     }
     else # first dir
     {
 	$dir_state->{dirbase} = pop @path;
 	$dir_state->{parent} = '';
+	$dir_state->{dir_url} = $self->{top_url};
     }
     # thumbnail dir for this directory
-    $dir_state->{abs_thumbdir} = File::Spec->catdir($dir_state->{abs_dir},
+    $dir_state->{abs_thumbdir} = File::Spec->catdir($dir_state->{abs_out_dir},
 	$self->{thumbdir});
 
     # reset the per-directory force_html flag
@@ -431,6 +457,11 @@ sub read_captions {
 
     my $captions_file = File::Spec->catfile($dir_state->{abs_dir},
 	$self->{captions_file});
+    if (!-f $captions_file)
+    {
+	$captions_file = File::Spec->catfile($dir_state->{abs_out_dir},
+					     $self->{captions_file});
+    }
     if (-f $captions_file)
     {
 	$dir_state->{captions} = {};
@@ -440,24 +471,21 @@ sub read_captions {
 
 =head2 read_dir
 
-Read the $dir_state->{dir} directory.
-Sets $dir_state->{subdirs}, $dir_state->{index_files} and $dir_state->{files}
-with the relative subdirs, index*.html files, and other files.
+Read the $dir_state->{dir} directory.  Sets $dir_state->{subdirs}, and
+$dir_state->{files} with the relative subdirs, and other files.
 
 =cut
 sub read_dir {
     my $self = shift;
     my $dir_state = shift;
 
-    my $abs_dir = $dir_state->{abs_dir};
     my $dh;
-    opendir($dh, $abs_dir) or die "Can't opendir $abs_dir: $!";
+    opendir($dh, $dir_state->{abs_dir}) or die "Can't opendir $dir_state->{abs_dir}: $!";
     my @subdirs = ();
     my @files = ();
-    my @index_files = ();
     while (my $fn = readdir($dh))
     {
-	my $abs_fn = File::Spec->catfile($abs_dir, $fn);
+	my $abs_fn = File::Spec->catfile($dir_state->{abs_dir}, $fn);
 	if ($fn =~ /^\./ or $fn eq $self->{thumbdir})
 	{
 	    # skip
@@ -466,10 +494,9 @@ sub read_dir {
 	{
 	    push @subdirs, $fn;
 	}
-	# remember the index files
-	elsif ($fn =~ /index.*\.html$/)
+	# ignore any html files
+	elsif ($fn =~ /\.html$/)
 	{
-	    push @index_files, $fn;
 	}
 	else
 	{
@@ -477,11 +504,45 @@ sub read_dir {
 	}
     }
     closedir($dh);
+
     $dir_state->{subdirs} = \@subdirs;
     $dir_state->{files} = \@files;
-    $dir_state->{index_files} = \@index_files;
 } # read_dir
 
+=head2 read_out_dir
+
+Read the $dir_state->{dir} directory in the output tree.
+Sets $dir_state->{index_files} with the index*.html files.
+
+=cut
+sub read_out_dir {
+    my $self = shift;
+    my $dir_state = shift;
+
+    my @index_files = ();
+    if (-d $dir_state->{abs_out_dir})
+    {
+	my $dh;
+	opendir($dh, $dir_state->{abs_out_dir}) or die "Can't opendir $dir_state->{abs_out_dir}: $!";
+	while (my $fn = readdir($dh))
+	{
+	    my $abs_fn = File::Spec->catfile($dir_state->{abs_out_dir}, $fn);
+	    if ($fn =~ /^\./ or $fn eq $self->{thumbdir})
+	    {
+		# skip
+	    }
+	    # remember the index files
+	    elsif ($fn =~ /index.*\.html$/)
+	    {
+		push @index_files, $fn;
+	    }
+	}
+	closedir($dh);
+    }
+
+    $dir_state->{index_files} = \@index_files;
+} # read_out_dir
+
 =head2 filter_images
 
 Sets $dir_state->{files} to contain only image files that
@@ -576,6 +637,12 @@ sub make_index_page {
     }
     $dir_state->{pages} = $pages;
 
+    # make the output dir if it doesn't exist
+    if (!-d $dir_state->{abs_out_dir})
+    {
+	mkdir $dir_state->{abs_out_dir};
+    }
+
     # if we have any new images in this directory, we need to re-make the index
     # files because we don't know which index file it will appear in,
     # and we need to re-make the other HTML files because
@@ -589,7 +656,7 @@ sub make_index_page {
     {
 	foreach my $if (@{$dir_state->{index_files}})
 	{
-	    my $ff = File::Spec->catfile($dir_state->{abs_dir}, $if);
+	    my $ff = File::Spec->catfile($dir_state->{abs_out_dir}, $if);
 	    unlink $ff;
 	}
     }
@@ -671,7 +738,7 @@ sub clean_thumb_dir {
     my $self = shift;
     my $dir_state = shift;
 
-    my $dir = File::Spec->catdir($dir_state->{abs_dir}, $self->{thumbdir});
+    my $dir = File::Spec->catdir($dir_state->{abs_out_dir}, $self->{thumbdir});
     my @pics = @{$dir_state->{files}};
     $self->debug(2, "dir: $dir");
 
@@ -813,7 +880,8 @@ sub make_thumbnail {
 	mkdir $dir_state->{abs_thumbdir};
     }
 
-    my ($x, $y) = dim($img_state->{info});
+    my $x = $img_state->{info}->{ImageWidth};
+    my $y = $img_state->{info}->{ImageHeight};
     if (!$x or !$y)
     {
 	warn "dimensions of " . $img_state->{abs_img} . " undefined -- faking it";
@@ -827,8 +895,18 @@ sub make_thumbnail {
     my $newx = int($x / (sqrt($x * $y) / sqrt($self->{pixelcount})));
     my $newy = int($y / (sqrt($x * $y) / sqrt($self->{pixelcount})));
     my $newpix = $newx * $newy;
-    my $command = "convert -geometry \"${newx}x${newy}\>\" \"$img_state->{abs_img}\" \"$thumb_file\"";
-    system($command);
+    my $command = '';
+    if ($img_state->{cur_img} =~ /\.gif$/)
+    {
+	# in case this is an animated gif, get the first frame only
+	$command = "convert -geometry \"${newx}x${newy}\>\" \"$img_state->{abs_img}\[0\]\" \"$thumb_file\"";
+    }
+    else
+    {
+	$command = "convert -geometry \"${newx}x${newy}\>\" \"$img_state->{abs_img}\" \"$thumb_file\"";
+    }
+    system($command) == 0
+	or die "$command failed";
     
 } # make_thumbnail
 
@@ -1232,7 +1310,7 @@ sub get_index_pagename {
     
     if ($args{get_filename})
     {
-	return File::Spec->catfile($dir_state->{abs_dir}, $pagename);
+	return File::Spec->catfile($dir_state->{abs_out_dir}, $pagename);
     }
     else # get URL
     {
@@ -1270,7 +1348,7 @@ sub get_image_pagename {
     $img_page .= ".html";
     if ($args{type} eq 'file')
     {
-	return File::Spec->catfile($dir_state->{abs_dir}, $thumbdir, $img_page);
+	return File::Spec->catfile($dir_state->{abs_out_dir}, $thumbdir, $img_page);
     }
     elsif ($args{type} eq 'parent')
     {
@@ -1316,11 +1394,11 @@ sub get_thumbnail_name {
     my $thumbdir = $self->{thumbdir};
     my $thumb = $image;
     # change the last dot to underscore
-    $thumb =~ s/\.(\w+)$/_$1/;
+    $thumb =~ s/\.([\w]+)$/_$1/;
     $thumb .= ".jpg"; 
     if ($args{type} eq 'file')
     {
-	return File::Spec->catfile($dir_state->{abs_dir}, $thumbdir, $thumb);
+	return File::Spec->catfile($dir_state->{abs_out_dir}, $thumbdir, $thumb);
     }
     elsif ($args{type} eq 'parent')
     {
@@ -1599,11 +1677,16 @@ sub make_image_content {
     my $caption = $self->get_caption(dir_state=>$dir_state,
 				     img_state=>$img_state,
 				     image=>$img_name);
+    my $img_url = "../$img_name";
+    if ($self->{top_dir} ne $self->{top_out_dir})
+    {
+	$img_url = $dir_state->{dir_url} . '/' . $img_name;
+    }
     my @out = ();
     push @out, "<div class=\"image\">\n";
-    my $width = $img_state->{info}->{width};
-    my $height = $img_state->{info}->{height};
-    push @out, "<img src=\"../$img_name\" alt=\"$img_name\" style=\"width: ${width}px; height: ${height}px;\"/>\n";
+    my $width = $img_state->{info}->{ImageWidth};
+    my $height = $img_state->{info}->{ImageHeight};
+    push @out, "<img src=\"$img_url\" alt=\"$img_name\" style=\"width: ${width}px; height: ${height}px;\"/>\n";
     push @out, "<p class=\"caption\">$caption</p>\n";
     push @out, "</div>\n";
     return join('', @out);
@@ -1671,7 +1754,7 @@ sub images_added_or_gone {
     my $self = shift;
     my $dir_state = shift;
 
-    my $dir = File::Spec->catdir($dir_state->{abs_dir}, $self->{thumbdir});
+    my $dir = File::Spec->catdir($dir_state->{abs_out_dir}, $self->{thumbdir});
     my @pics = @{$dir_state->{files}};
     $self->debug(2, "dir: $dir");
 
@@ -1762,7 +1845,7 @@ sub get_image_info {
     my $self = shift;
     my $img_file = shift;
 
-    my $info = image_info($img_file);
+    my $info = Image::ExifTool::ImageInfo($img_file);
     # add the basename
     my ($basename, $path, $suffix) = fileparse($img_file, qr/\.[^.]*/);
     $info->{file_basename} = $basename;
